
type motion =
  | Periodic of float * float
  | Rotation of float * float * float * float
  | BrowinanMotion

let pi = 4. *. atan 1.
let circle_point (x, y) r n_max idx =
  let theta = 2. *. pi *. (float idx) /. (float n_max) in
  (x +. r *. cos theta, y +. r *. sin theta)

let circle_points (x, y) = Array.init 31 (circle_point (x, y) 1. 31)



let rec box_muller () =
  let (u, v) = (Random.float 1., Random.float 1.) in
  let delta_x = sqrt (-2. *. log u) *. cos (2. *. pi *. v) in
  let delta_y = sqrt (-2. *. log u) *. sin (2. *. pi *. v) in
  (delta_x, delta_y)


let x = ref 0.
let y = ref 0.
let t = ref 0.

let () =
  Random.self_init ();
  ignore (Glut.init Sys.argv);
  Glut.initDisplayMode ~double_buffer:true ();
  Glut.initWindowSize ~w:512 ~h:512;
  ignore (Glut.createWindow ~title:"LablGL Brownian Motion Demo");
  GlClear.color (0.6, 0.6, 1.0);
  GlDraw.shade_model `smooth;
	
  let dp m h =
    let (delta_x, delta_y) = box_muller () in
    match m with
    | Periodic (a,f0) (* 振幅a, 振動数 f0 の速度を実装しましょう *)
      -> (0., 0.)
    | Rotation (ax,ay,bx,by) -> ((-.2. *. pi *. ax *. sin (2. *. pi *. !t) -. 2. *. pi *. bx *. sin (100. *. pi *. !t))*. h, (2. *. pi *. ay *. cos (2. *. pi *. !t) +. 2. *. pi *. by *. cos (100. *. pi *. !t))*. h)
    | BrowinanMotion -> (delta_x *. (sqrt h), delta_y *. (sqrt h))
  in

  let render m =
    GlMat.mode `projection;
    GlMat.load_identity ();
    GluMat.ortho2d ~x:(-10., 10.) ~y:(-10., 10.);
    GlMat.mode `modelview;
    GlMat.load_identity ();
    GlClear.clear [ `color ];

    (* 振動 *)
    (* let (delta_x, delta_y) = dp (Periodic (1.0,1.0)) 0.0001 in *)
    let (delta_x, delta_y) = dp (Rotation (4.,2.,8.,4.)) 0.0001 in *)
    (* ブラウン運動 *)
    (* let (delta_x, delta_y) = dp BrowinanMotion 0.0001 in *)
    x := !x +. delta_x;
    y := !y +. delta_y;
    t := if !t < 1.0 then !t +. 0.0001 else 0.;

    GlDraw.begins `polygon;
    Array.iter GlDraw.vertex2 (circle_points (!x, !y));
    GlDraw.ends ();
    Gl.flush ();
    Glut.swapBuffers ()
  in
  Glut.displayFunc ~cb:render;
  Glut.idleFunc ~cb:(Some Glut.postRedisplay);
  Glut.keyboardFunc ~cb:(fun ~key ~x ~y -> if key=27 then exit 0);
  Glut.mainLoop ()
