
(* パイプライン演算子 *)
let (|>) x f = f x

(* 一様乱数から正規乱数を作成する *)
let rand () = Random.float 1.0
let randn () =
  (Array.init 12 (fun n -> Random.float 1.0) |> Array.fold_left (+.) 0.) -. 6. 

(* ガウス分布に従う測定値のばらつき *)
let volt voltage temp_variance =
  randn () *. temp_variance +. voltage
    
(* 1D カルマンフィルタ *)
class kalmanfilter1d (x0 : float) (p0 : float) (r0 : float) (q0 : float) =
  object (self)
    val mutable x = x0
    val mutable p = p0
    val r = r0
    val q = q0

    method update z =
      x <- (p *. z +. x *. r) /. (p +. r);
      p <- 1. /. (1. /. p +. 1. /. r)

    method predict u =
      x <- x +. u;
      p <- p +. q

    method pos () = x
  end


    
let () =
  let movement = 0. in
  let variance = 2.13 ** 2. in
  let movement_variance = 0.2 in
  let actual_voltage = 16.3 in

  let n = 50 in
  let zs = Array.init n (fun k -> volt actual_voltage variance) in
 
  let kf = new kalmanfilter1d  25. 1000. variance movement_variance in
  for i=0 to n-1 do
    kf#predict movement;
    kf#update zs.(i);
    print_int i;
    print_string " ";
    print_float (kf#pos ());
    print_string " ";
    print_float zs.(i);
    print_endline "";
  done;
  ()
