(*
  簡易 ODS アニメーション
 *)


let init_gl width height =
  GlDraw.shade_model `smooth;
  GlClear.color (0.0, 0.0, 0.0);
  GlClear.depth 1.0;
  GlClear.clear [`color; `depth];
  Gl.enable `depth_test;
  GlFunc.depth_func `lequal;
  GlMisc.hint `perspective_correction `nicest


let pi = 4. *. atan 1.

let theta = ref 0.
let dx1 = ref 0.
let dz1 = ref 0.
let dx2 = ref 0.
let dz2 = ref 0.

let elastic x y z angle hx hz =
  GlMat.load_identity ();
  GlMat.translate3 (x, y, z);
  GlMat.rotate3 angle (0.0, 1.0, 0.0);
  GlDraw.begins `quads;

  GlDraw.color (0.5,0.5,0.5);                
  GlDraw.vertex3 ( 1.0 +. hx, 1.0,-.0.2 +. hz);
  GlDraw.vertex3 (-1.0 +. hx, 1.0,-.0.2 +. hz);
  GlDraw.vertex3 (-1.0 +. hx, 1.0, 0.2 +. hz);
  GlDraw.vertex3 ( 1.0 +. hx, 1.0, 0.2 +. hz);  
  
  (* 底面 : 振動しない *)
  GlDraw.color (0.5,0.5,0.5);                 
  GlDraw.vertex3 ( 1.0,-1.0, 0.2);
  GlDraw.vertex3 (-1.0,-1.0, 0.2);
  GlDraw.vertex3 (-1.0,-1.0,-.0.2);
  GlDraw.vertex3 ( 1.0,-1.0,-.0.2);  
  
  GlDraw.color (0.5,0.5,0.5);                   
  GlDraw.vertex3 ( 1.0 +. hx, 1.0, 0.2 +. hz);
  GlDraw.vertex3 (-1.0 +. hx, 1.0, 0.2 *. hz);
  GlDraw.vertex3 (-1.0,-1.0, 0.2);
  GlDraw.vertex3 ( 1.0,-1.0, 0.2);
  
  GlDraw.color (0.5,0.5,0.5);                   
  GlDraw.vertex3 ( 1.0,-1.0,-.0.2); 
  GlDraw.vertex3 (-1.0,-1.0,-.0.2); 
  GlDraw.vertex3 (-1.0 +. hx, 1.0,-.0.2 +. hz); 
  GlDraw.vertex3 ( 1.0 +. hx, 1.0,-.0.2 +. hz); 
  
  GlDraw.color (0.5,0.5,0.5);                   
  GlDraw.vertex3 (-1.0 +. hx, 1.0, 0.2 +. hz); 
  GlDraw.vertex3 (-1.0 +. hx, 1.0,-.0.2 +. hz); 
  GlDraw.vertex3 (-1.0,-1.0,-.0.2); 
  GlDraw.vertex3 (-1.0,-1.0, 0.2); 
  
  GlDraw.color (0.5,0.5,0.5);   
  GlDraw.vertex3 ( 1.0 +. hx, 1.0,-.0.2 +. hz); 
  GlDraw.vertex3 ( 1.0 +. hx, 1.0, 0.2 +. hz); 
  GlDraw.vertex3 ( 1.0,-1.0, 0.2); 
  GlDraw.vertex3 ( 1.0,-1.0,-.0.2); 

  GlDraw.ends ()

    
let draw_gl_scene () =
  GlClear.clear [`color; `depth];
  elastic 0. 0. (-7.) 45. !dx1 !dz1;
  elastic (-3.) 0. (-10.) 45. !dx2 !dz2; 
  Glut.swapBuffers ();
  theta := !theta +. 0.001 *. pi;
  dx1 := 0.1 *. (sin !theta);
  (* dz1 := 0.1 *. (sin !theta); *) 
  dx2 := 0.05 *. (sin !theta);
  (* dz2 := 0.05 *. (sin (!theta +. pi /. 2.)); *)
  ()

(* Handle window reshape events *)
let reshape_cb ~w ~h =
  let ratio = (float_of_int w) /. (float_of_int h) in
  GlDraw.viewport 0 0 w h;
  GlMat.mode `projection;
  GlMat.load_identity ();
  GluMat.perspective 45.0 ratio (0.1, 100.0);
  GlMat.mode `modelview;
  GlMat.load_identity ()

(* Handle keyboard events *)
let keyboard_cb ~key ~x ~y =
  match key with
  | 27 (* ESC *) -> exit 0
  | _ -> ()

(* Draw the scene whever idle *)
let idle_cb () =
  draw_gl_scene ()

let () =
  let width = 640 and height = 480 in
  ignore (Glut.init Sys.argv);
  Glut.initDisplayMode ~alpha:true ~depth:true ~double_buffer:true ();
  Glut.initWindowSize width height;
  ignore (Glut.createWindow "LablGL vibration");
  Glut.displayFunc draw_gl_scene;
  Glut.keyboardFunc keyboard_cb;
  Glut.reshapeFunc reshape_cb;
  Glut.idleFunc(Some idle_cb);
  init_gl width height;
  Glut.mainLoop ()
