
open Realext
open Linear
open Diffeq


let l1 = 0.5
let m1 = 1.
let l2 = 0.5
let m2 = 1.

let g = 9.81

(* Setting Parameters *)
let l = 0.5
let m = 1.
let g = 9.81

let (x1o, y1o) = (-.l, 0.)
let (x1a, y1a) = (l, 0.)

(* 加速度の計算 *)
let acc (q,v) =
  let sq = sin (q.(2).(0)) in
  let cq = cos (q.(2).(0)) in

  (*
    M  Φq
    Φq 0
    の matrix を計算する
   *)
  let c_mat = [|
    [| m; 0.; 0.; 1.; 0. |];
    [| 0.; m; 0.; 0.; 1. |];
    [| 0.; 0.; m *.(l *^ 2) /. 3.; l*.sq; -.l*.cq|];
    [| 1.; 0.; l*.sq; 0.; 0. |];
    [| 0.; 1.; -.l*.cq; 0.; 0. |];
  |] in
  let b = [|
    [| 0. |];
    [| -.m*.g |];
    [| 0. |];
    [| -.l*.(v.(2).(0) *^ 2)*.cq |];
    [| -.l*.(v.(2).(0) *^ 2)*.sq |]
  |] in
  let a_ext = solve c_mat b in
  Array.sub a_ext 0 3

(* 正規形微分方程式を適用する場合 *)
let df acc (q,v) = (v, acc (q,v))

let dump_rigid_pend (q0, v0) t0 dt n_stop =
  let calc_point q =
    let cq = cos (q.(2).(0)) in
    let sq = sin (q.(2).(0)) in
    (q.(0).(0) +. (x1a *. cq -. y1a *. sq), q.(1).(0) +. (x1a *. sq +. y1a *. cq))
  in

  let rec dump_next_point (q_pre, v_pre) t0 idx =
    let (q_cur, v_cur) =runge_kutta_acc acc (q_pre, v_pre) dt in
    let (xp, yp) = calc_point q_cur in
    let t_cur = t0 +. (float_of_int (idx+1)) *. dt in
    print_endline ((string_of_float t_cur) ^ " " ^ (string_of_float xp) ^ " " ^ (string_of_float yp));
    if (idx + 1) >= n_stop then ()
    else dump_next_point (q_cur, v_cur) t0 (idx+1)
  in

  let (xp_0, yp_0) = calc_point q0 in
  print_endline ("0. " ^ (string_of_float xp_0) ^ " " ^ (string_of_float yp_0));
  dump_next_point (q0, v0) 0. 0

let () =
  (* 初期条件 *)
  let theta0 = -.pi /. 4. in
  let q0 = [| [| l *. (cos theta0) |]; [| l *. (sin theta0) |]; [| theta0 |] |] in
  let v0 = [| [| 0. |]; [| 0. |]; [| 0. |] |] in
  
  dump_rigid_pend (q0, v0) 0. 0.001 5000 

