open Linear

(* Euler method *)
let euler_acc df (q_pre, v_pre) dt =
  let a_pre = df (q_pre, v_pre) in
  let y_pre = Array.append q_pre v_pre in
  let yd_pre = Array.append v_pre a_pre in
  let y = y_pre +! (dt *< yd_pre) in
  let q_cur = Array.sub y 0 3 in
  let v_cur = Array.sub y 3 3 in
  (q_cur, v_cur)

(* Runge-Kutta method *)
let runge_kutta_acc dv (q_pre,v_pre) dt =
  let k1_q = v_pre in
  let k1_v = dv (q_pre, v_pre) in
  
  let k2_q = v_pre +! ((0.5 *. dt) *< k1_v) in
  let k2_v = dv (q_pre +! ((0.5 *. dt) *< k1_q), v_pre +! ((0.5 *. dt) *< k1_v)) in
  
  let k3_q = v_pre +! ((0.5 *. dt) *< k2_v) in
  let k3_v = dv (q_pre +! ((0.5 *. dt) *< k2_q), v_pre +! ((0.5 *. dt) *< k2_v)) in

  let k4_q = v_pre +! (dt *< k3_v) in
  let k4_v = dv (q_pre +! (dt *< k3_q), v_pre +! (dt *< k3_v)) in

  let q_cur = q_pre +! ((dt /. 6.) *< (k1_q +! (2. *< k2_q) +! (2. *< k3_q) +! k4_q)) in
  let v_cur = v_pre +! ((dt /. 6.) *< (k1_v +! (2. *< k2_v) +! (2. *< k3_v) +! k4_v)) in
  (q_cur, v_cur)
