(*
  Newmark Beta 法 :
 *)
let beta = 0.25
let gamma = 0.5


let n_smp_1sec = 10000
let n_sec = 1
let n_smp = n_smp_1sec * n_sec
let dt = 1. /. (float_of_int n_smp_1sec)


let newmark_beta_1d (m, c, k) fn (x_p, v_p, a_p) =
  let v_tmp = v_p +. dt *. (1. -. gamma) *. a_p in
  let x_tmp = x_p +. dt *. v_p +. 0.5 *. dt *. dt *. (1. -. 2.*.beta) *. a_p in
  let b_tmp = fn -. c *. v_tmp -. k *. x_tmp in
  let a_c = b_tmp /. (m +. ((dt *. gamma)*. c) +. ((dt *. dt *. beta) *. k)) in
  let v_c = v_p +. dt *. ((1. -. gamma) *. a_p +. gamma *. a_c) in
  let x_c = x_p +. dt *. v_p +. 0.5 *. dt *. dt *. (1. -. 2.*.beta) *. a_p +. beta *. dt *. dt *. a_c in
  (x_c, v_c, a_c)

(* 
   Newmark Beta 法の計算結果を表示 
   ~m : 質量行列
   ~c : 摩擦行列
   ~k : 剛性行列
   ~x0 : 位置初期条件
   ~v0 : 速度初期条件
   ~f_ext : 外力
   設定する順番は自由
*)
let newmark_beta_result ~m ~c ~k ~x0 ~v0 ~f_ext =
  (* 前回値→今回値をサンプル数分プロットする内部関数 *)
  let rec dump_newmark_beta n_cur f (x_p, v_p, a_p) =
    let t = (float_of_int n_cur) /. (float_of_int n_smp_1sec) in
    let (x_c, v_c, a_c) = newmark_beta_1d (m, c, k) (f t) (x_p, v_p, a_p) in

    (* 結果のプロット *)
    print_string ((string_of_float t) ^ "\t");
    print_string ((string_of_float v_c) ^ "\t");
    print_endline "";
    if n_cur + 1 >= n_smp then () else dump_newmark_beta (n_cur+1) f (x_c, v_c, a_c)
  in

  (* 初期条件の設定 *)
  let f0 = f_ext 0. in
  let b0 = f0 -. c *.v0 -. k *.x0 in
  (* 加速度の計算 *)
  let a0 = b0 /. m in

  (* 初期条件を設定し, 内部関数による計算 *)
  dump_newmark_beta 1 f_ext (x0,v0,a0)


let pi = 4. *. atan 1.
(* main *)
let () =
  (* 質量行列 *)
  let m = 1. in

  (* 剛性行列 *)
  let k = 100. in

  (* 摩擦抵抗 *)
  let c = 1. in

  (* 微分方程式の初期条件 *)
  let x0 = 0. in

  let v0 = 0. in

  let ext_force t = 14.1 *. sin (120. *. pi *. t) in

  (*
    Newmark β 法による計算
    質量
    摩擦
    剛性
    初期条件(x0, v0)
    外力
    を登録して計算します
   *)
  newmark_beta_result ~m:m ~c:c ~k:k ~x0:x0 ~v0:v0 ~f_ext:ext_force

