Require Import ssreflect ssrnat ssrfun ssrbool eqtype.
Require Import List.


(* 取引のイベント *)
Inductive trade_event : Type :=
| Buy : nat -> nat -> trade_event
| Sell : nat -> nat -> trade_event.


(* 製品の在庫と資金  *)
Record status := { stock : nat; money : nat }.

(* 1つの取引イベント後の状態 *)
Definition next_status (cur : status) (tev : trade_event) : status :=
  match tev with
    | Buy num price => if money cur >= price then Build_status (stock cur + num) (money cur - num*price) else cur
    | Sell num price => if num <= stock cur then Build_status (stock cur - num) (money cur + num*price) else cur
  end.


Definition status_init := Build_status 0 5000.
Compute next_status status_init (Buy 5 100).  

Fixpoint last_status (s0 : status) (tl : list trade_event) : status :=
  match tl with
    | nil => s0
    | t :: ts => last_status (next_status s0 t) ts
  end.


Lemma addn_comm : forall n m, n + m = m + n.
Proof.
  move => n m.
  ring.
Qed.

(*
  浅井 琢さんの修士論文から引用
  自然数の引き算の分配法則
 *)
Lemma mult_div : forall a b c, a * c - b * c = (a - b) *  c.
Proof.
  move => a b c.
  case le : (a <= b).
  rewrite -subn_eq0 in le.
  move /eqP in le.
  rewrite le /muln /= -/muln.
  apply /eqP.
  rewrite subn_eq0.
  move /eqP in le.
  rewrite subn_eq0 in le.
  by apply leq_mul.
  elim c.
  by rewrite !muln0.
  move => n IH.
  rewrite mulnSr mulnS mulnS subnDA -addnBA.
  rewrite addn_comm -IH addnBA //.
  apply leq_mul.
  move /negP in le.
  apply ltnW.
  rewrite leqNgt.
  by apply /negP.
  by [].
  move /negP in le.
  apply ltnW.
  rewrite leqNgt.
  by apply /negP.
Qed.

(*
　ssrnat を利用した証明
　マイナスがあると ring タクティックが使えない
 *)
Lemma pzero : forall (p : nat), p - 1 * p = 0.
Proof.
  move => p.
  (* p - 1 * p = 1 * p - 1 * p に変形する *)
  have Hp1 : (p = 1 * p).
  ring.
  rewrite {1} Hp1.
  rewrite mult_div.
  have Ho : (1 - 1 = 0).
  by [].
  rewrite Ho.
  ring.
Qed.


(* 1個仕入れて原価の3倍で販売したときの証明  *)
Lemma buysell_one_piece : forall (p : nat), last_status (Build_status 0 p) ((Buy 1 p) :: (Sell 1 (3*p)) :: nil) = Build_status 0 (3*p).
Proof.
  move => p.
  rewrite /last_status.
  rewrite /next_status.
  have Hleq : (p <= p).
  elim p.
  by [].
  move => n Hn.
  by [].
  rewrite Hleq.
  rewrite /money.
  rewrite /stock.
  have H01 : (0 < 0 + 1).
  by [].
  rewrite H01.
  have Ho : (0+1-1=0).
  by [].
  have Hp3 : (1 * (3 * p) = 3 * p).  
  ring.
  rewrite Ho.
  rewrite Hp3 pzero.
  have Hpz : (0 + 3 * p = 3 * p).
  by [].
  rewrite Hpz.
  by [].
Qed.