
(* Excel と OCaml の使用対応 *)

(* セルに入力する値 : Excel で扱われる値は OCaml では float に相当する *)
let sample = [| 1.; 2.; 3.; 4.; 5. |]

(*
  合計の値を計算
  OCaml の計算では float の足し算は ( +. ), ( + ) は int の足し算
  Array.fold_left とは data (配列)のすべての値に対して演算を行うときに利用する

  参考 : 
  data のすべての値に対して掛け算をする場合は Array.fold_right ( *. ) 1. data ... 
*)
let total data = Array.fold_left ( +. ) 0. data

(*
  平均を計算する関数
  関数は
  let 関数名 関数の評価に使われる変数(引数) = 評価される式
  で記述される.
  関数の評価に使われる変数が複数あるとき
  let func x y z = ...
  のようにスペースで区切って記述する
 *)
let mean data =
  (*
    平均を計算するために
   *)
  let n = Array.length data in (* 登録されたデータ数を求める : int 型 *)
  let s = total data in (* 登録されたデータの和を計算する : float 型 *)
  (*
    平均を計算する: OCaml では float と int の演算は禁止されているため
    int値 を float値に変換して計算する
   *)
  s /. (float n)

(* 
   分散の計算
   2乗平均 - 平均の2乗
 *)
let var data =
  (*
    2 乗平均
   *)
  let m2 = mean (Array.map (fun x -> x *. x) data) in
  let m = mean data in
  m2 -. m *. m
